# -*- coding: utf-8 -*-
"""
CIFAR.py
CIFAR10/100 : カラー画像分類
3x32x32
https://www.cs.toronto.edu/~kriz/cifar.html
"""

import os
from torchvision import datasets, transforms
import utils

# dataset
def dataset(DATA, MODEL, in_size):
    # dataset
    if DATA == 'CIFAR10':
        datasets_fn = datasets.CIFAR10
    elif DATA == 'CIFAR100':
        datasets_fn = datasets.CIFAR100
    else:
        print('invalid DATA : %s' % DATA)
        return

    # データファイルを保存する場所
    data_root = os.path.join(os.getcwd(), DATA)

    # 結果を図形表示するためのテストデータセット(加工しない:ToTensor()のみ)
    test0_set = datasets_fn(root=data_root, train=False, download=True, transform=transforms.ToTensor())

    # 平均と分散
    mean, std = utils.mean_std(test0_set, in_size[0])
    #print(mean, std)
    # CIFAR10  : [0.4942, 0.4851, 0.4504] [0.2466, 0.2428, 0.2615]
    # CIFAR100 : [0.5087, 0.4873, 0.4419] [0.2682, 0.2573, 0.2770]

    # リサイズ(必要なら変更する)
    resize = 112 if (MODEL == 2) or (MODEL == 3) else in_size[1]

    # 訓練データ変換
    transform_train = transforms.Compose([
        transforms.ToTensor(),  # 必須
        # 以下必要なら追加する(オプション)
        transforms.Resize(resize),        # リサイズ
        transforms.Normalize(mean, std),  # 正規化
        transforms.RandomHorizontalFlip(p=0.5),  # 左右反転
        #transforms.RandomErasing(p=0.5, scale=(0.02, 0.33), ratio=(0.3, 3.3), value=0, inplace=False) # 目隠し
    ])

    # テストデータ変換
    transform_test = transforms.Compose([
        transforms.ToTensor(),  # 必須
        # 以下必要なら追加する(オプション)
        transforms.Resize(resize),        # リサイズ
        transforms.Normalize(mean, std),  # 正規化
    ])

    # 訓練、テスト用データセット(50000, 10000)
    train_set = datasets_fn(root=data_root, train=True,  download=True, transform=transform_train)
    test_set  = datasets_fn(root=data_root, train=False, download=True, transform=transform_test)

    """
    # debug
    if DATA == 'CIFAR10':
        _plot_CIFAR10(test0_set)
    elif DATA == 'CIFAR100':
        _plot_CIFAR100(test0_set)
    """

    return train_set, test_set, test0_set

"""
# debug : テストデータセットの確認
import numpy as np
import matplotlib.pyplot as plt
import labels

def _plot_CIFAR10(dataset):
    strclasses = labels.get('CIFAR10')
    nrow = len(strclasses)
    ncol = 15

    plt.figure('CIFAR-10 dataset', figsize=(8, 6))
    plt.rcParams['font.family'] = 'MS gothic'

    ndone = np.zeros(nrow, 'i4')
    for (image, label) in dataset:
        assert label >= 0 and label < 10
        if ndone[label] < ncol:
            ipos = (ncol * label) + ndone[label] + 1
            ax = plt.subplot(nrow, ncol, ipos)
            image2 = np.transpose(image, (1, 2, 0))
            plt.imshow(image2)
            ax.set_axis_off()
            if ndone[label] == 0:
                ax.set_title(strclasses[label], fontsize=8, pad=0)
            ndone[label] += 1

def _plot_CIFAR100(dataset):
    strclasses = labels.get('CIFAR100')
    nrow = 10
    ncol = 10
    assert nrow * ncol == len(strclasses)

    plt.figure('CIFAR-100 dataset', figsize=(8, 7))
    plt.rcParams['font.family'] = 'MS gothic'

    idone = np.zeros(nrow * ncol, 'u1')
    ipos = 1
    for (image, label) in dataset:
        assert label >= 0 and label < len(strclasses)
        if idone[label] == 0:
            ax = plt.subplot(nrow, ncol, ipos)
            image2 = np.transpose(image, (1, 2, 0))
            plt.imshow(image2)
            ax.set_axis_off()
            ax.set_title(strclasses[label], fontsize=8, pad=0)
            ipos += 1
            idone[label] = 1

if __name__ == "__main__":
    dataset('CIFAR100', 0, (3, 32, 32))
"""
