# -*- coding: utf-8 -*-
"""
CNN.py
CNNによる画像分類 (PyTorch)
"""

import torch
from torch import nn, optim
from torchvision import models
#from torchinfo import summary

import utils, fit, myCNN, myResNet, labels
import MNIST, EMNIST, CIFAR, Flowers102
import K49, Kanji

def main():

    # データセット名、使用するDATAをコメントOFFにする
    # torchvisionがサポートするデータセット
    # https://pytorch.org/vision/stable/datasets.html
    DATA = 'MNIST'
    #DATA = 'QMNIST'
    #DATA = 'EMNIST'
    #DATA = 'FashionMNIST'
    #DATA = 'KMNIST'
    #DATA = 'CIFAR10'
    #DATA = 'CIFAR100'
    #DATA = 'Flowers102'

    # torchvisionがサポートしないデータセット
    #DATA = 'K49'          # Kuzushiji-49
    #DATA = 'KKanji'       # Kuzushiji-Kanji
    #DATA = 'TKanji'       # Kanji

    # device
    device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
    #device = torch.device('cpu')  # CPU強制

    # 計算パラメーター(要編集)
    MODEL = 0          # 0:自作CNN, 1:自作ResNet, 2:公開ResNet, 3:VGG
    restart = False    # リスタート: 前回の結果から継続するにはTrueにする(同じDATA,MODEL限定)
    num_epochs = 5   # 繰り返し回数
    batch_size = 50    # ミニバッチサイズ
    mixed = True and (str(device) != 'cpu')  # 混合精度: 画像サイズが大きいとき速くなる(GPU限定)

    # header表示
    print('%s, %s (device=%s)' % (DATA, ['自作CNN', '自作ResNet', '公開ResNet', 'VGG'][MODEL], device))

    # 乱数初期化
    utils.torch_seed()

    # 画像サイズ, dataset
    if   DATA == 'MNIST' or \
         DATA == 'QMNIST' or \
         DATA == 'FashionMNIST' or \
         DATA == 'KMNIST':
        in_size = (1, 28, 28)
        train_set, test_set, test0_set = MNIST.dataset(DATA, MODEL, in_size)
    elif DATA == 'EMNIST':
        in_size = (1, 28, 28)
        train_set, test_set, test0_set = EMNIST.dataset(MODEL, in_size)
    elif DATA == 'CIFAR10' or \
         DATA == 'CIFAR100':
        in_size = (3, 32, 32)
        train_set, test_set, test0_set = CIFAR.dataset(DATA, MODEL, in_size)
    elif DATA == 'Flowers102':
        in_size = (3, 32, 32)#300, 300)
        train_set, test_set, test0_set = Flowers102.dataset(in_size, False, False, 1)
    elif DATA == 'K49':
        in_size = (1, 28, 28)
        train_set, test_set, test0_set = K49.dataset(MODEL, in_size)
    elif DATA == 'KKanji' or \
         DATA == 'TKanji':
        in_size = (1, 64, 64)
        train_set, test_set, test0_set = Kanji.dataset(DATA, MODEL, in_size)
    else:
        print('変数DATAが正しくありません。')
        return
    #print(len(train_set), len(test_set), len(test0_set))

    # クラスラベル文字列
    strclasses = labels.get(DATA)
    #print(len(strclasses))
    #print(strclasses)

    # debug : テストデータセットの確認
    #color = (DATA == 'CFAR10') or (DATA == 'CFAR100') or (DATA == 'Flowers32')
    #transpose = (DATA == 'EMNIST')
    #utils.plot_dataset(train_set, strclasses, w=9.5, h=7.5, nrow=10, ncol=10,
    #    startlabel=0, stoplabel=10, fontsize=8, color=color, transpose=transpose)
    #utils.plot_MNIST(DATA, test0_set, strclasses, 0, 10, 15, w=9, h=6, name=False, transpose=transpose)

    # dataloader
    train_loader, test_loader = utils.dataloader(train_set, test_set, batch_size=batch_size)

    # model(要編集)
    if   MODEL == 0: # 自作CNN
        # 固定
        #net = myCNN.CNN2(in_size, [32], [0.3], len(strclasses))
        #net = myCNN.CNN4(in_size, [64], [0.3], len(strclasses))
        net = myCNN.CNN6(in_size, [64], [0.3], len(strclasses))
        #net = myCNN.CNN8(in_size, [128], [0.3], len(strclasses))
        #net = myCNN.CNN10(in_size, [128], [0.3], len(strclasses))
        # 可変(通常使用しない)
        #net = myCNN.CNN2(in_size, [32, 32], [0.4, 0.5], len(strclasses))
        #net = myCNN.CNN4(in_size, [32, 32, 64, 64], [0.3, 0.4, 0.5], len(strclasses))
        #net = myCNN.CNN6(in_size, [32, 32, 64, 64, 128, 128], [0.2, 0.3, 0.4, 0.5] len(strclasses))
        #net = myCNN.CNN8(in_size, [32, 32, 64, 64, 128, 128, 192, 192], [0.1, 0.2, 0.3, 0.4, 0.5], len(strclasses))
        #net = myCNN.CNN10(in_size, [32, 32, 64, 64, 96, 96, 128, 128, 160, 160], [0.1, 0.2, 0.3, 0.4, 0.5, 0.6], len(strclasses))
    elif MODEL == 1: # 自作ResNet
        net = myResNet.ResNet(in_size[0], 4, 128, 0.3, len(strclasses))
    elif MODEL == 2: # 公開ResNet
        #net = models.resnet18(weights=None)  # 重みなし
        net = models.resnet18(weights=models.ResNet18_Weights.DEFAULT)
        #net = models.resnet34(weights=models.ResNet34_Weights.DEFAULT)
        #net = models.resnet50(weights=models.ResNet50_Weights.DEFAULT)
        net.fc = nn.Linear(net.fc.in_features, len(strclasses))  # 最終レイヤー関数の付け替え
    elif MODEL == 3: # VGG
        net = models.vgg11_bn(weights=models.VGG11_BN_Weights.DEFAULT)
        #net = models.vgg13_bn(weights=models.VGG13_BN_Weights.DEFAULT)
        #net = models.vgg16_bn(weights=models.VGG16_BN_Weights.DEFAULT)
        #net = models.vgg19_bn(weights=models.VGG19_BN_Weights.DEFAULT)
        net.classifier[6] = nn.Linear(net.classifier[6].in_features, len(strclasses))  # 最終レイヤー関数の付け替え
    #print(net)
    #summary(net, (50, 1, 28, 28))

    # restart（前回と同じDATA,MODELのとき有効）
    modelfile = '%s.pth' % DATA
    if restart:
        utils.load_model(net, modelfile, device)

    # 損失関数: 交差エントロピー関数
    criterion = nn.CrossEntropyLoss()

    # 最適化関数(要編集)
    #optimizer = optim.SGD(net.parameters())
    #optimizer = optim.SGD(net.parameters(), lr=0.001, momentum=0.9)
    optimizer = optim.Adam(net.parameters())

    # 計算条件表示
    print('train: %d, test: %d, classes: %d, epochs: %d' % (len(train_set), len(test_set), len(strclasses), num_epochs))

    # 学習実行
    history, test_labels = fit.fit(net, device, num_epochs, train_loader, test_loader, optimizer, criterion, modelfile, mixed=mixed)

    # 計算経過を図形表示する
    #utils.show_history(history)

    # 画像を図形表示する
    #_plot_images(DATA, test0_set, test_labels, strclasses)

    # debug: クラス別の正解率を出力する(最小データ数指定:KKanji用)
    #num_data = utils.num_data_class(test0_set, test_labels, strclasses)
    #utils.output_accuracy(test0_set, test_labels, strclasses, num_data, -1)  # debug
    #utils.output_accuracy(test0_set, test_labels, strclasses, num_data, 0)
    #utils.output_accuracy(test0_set, test_labels, strclasses, num_data, 5)
    #utils.output_accuracy(test0_set, test_labels, strclasses, num_data, 10)
    #utils.output_class_html(strclasses, num_data, 10)  # HTML (KKanji用)

# 画像を図形表示する
def _plot_images(DATA, test0_set, test_labels, strclasses):
    num_data = utils.num_data_class(test0_set, test_labels, strclasses)

    # パラメーター
    w, h = 9.5, 6.5
    nrow, ncol = 10, 20
    step = 1
    thres = 0
    fontsize = 8
    chars = -1
    # 図形表示パラメーター
    fontname = 'MS gothic' if (DATA == 'KMNIST') or (DATA == 'CIFAR10') or (DATA == 'CIFAR100') or (DATA == 'K49') or (DATA == 'KKanji') or (DATA == 'TKanji') or (DATA == 'Flowers102') else None
    cmap = 'gray_r' if (DATA == 'MNIST') or (DATA == 'QMNIST') or (DATA == 'EMNIST') or (DATA == 'FashionMNIST') or (DATA == 'KMNIST') else 'gray' if (DATA == 'K49') or (DATA == 'KKanji') or (DATA == 'TKanji') else None
    transpose = (DATA == 'EMNIST')
    # 全データ
    utils.show_images_labels('CNN-1', test0_set, test_labels, strclasses, num_data,
        w=w, h=h, fontname=fontname, fontsize=fontsize, nrow=nrow, ncol=ncol, chars=chars,
        incorrect=False, start=0, step=step, thres=thres, cmap=cmap, transpose=transpose)
    # 不正解のみ
    utils.show_images_labels('CNN-2', test0_set, test_labels, strclasses, num_data,
        w=w, h=h, fontname=fontname, fontsize=fontsize, nrow=nrow, ncol=ncol, chars=chars,
        incorrect=True,  start=0, step=step, thres=thres, cmap=cmap, transpose=transpose)

if __name__ == "__main__":
    main()
