# -*- coding: utf-8 -*-
"""
EMNIST.py
手書き英数字: 0-9, A-Z, a-z
1x28x28
https://arxiv.org/pdf/1702.05373v1
"""

import os
import torch
from torchvision import datasets, transforms

# dataset
def dataset(MODEL, in_size):
    # 前処理
    if (MODEL == 0) or (MODEL == 1):
        transform = transforms.Compose([
            transforms.ToTensor(),  # 必須
            transforms.Lambda(lambda x: 1 - x),  # 白黒反転
            # 以下必要なら追加する
            #transforms.Normalize(0.5, 0.5),  # 正規化(EMNISTでは効果ない?)
        ])
    else:
        transform = transforms.Compose([
            transforms.ToTensor(),  # 必須
            transforms.Lambda(lambda x: 1 - x),  # 白黒反転
            transforms.Lambda(lambda x: torch.cat((x, x, x), 0).view(-1, in_size[1], in_size[2])), # gray->RGB
            # 以下必要なら追加する
            #transforms.Resize(112),  # サイズ拡大(計算時間が増える)
            #transforms.Normalize(0.5, 0.5),  # 正規化(EMNISTでは効果ない?)
        ])

    # データファイルを保存する場所
    data_root = os.getcwd()

    # 訓練、テスト用データセット(balanced : 112800, 18800)
    train_set = datasets.EMNIST(root=data_root, split='balanced', train=True,  download=True, transform=transform)
    test_set  = datasets.EMNIST(root=data_root, split='balanced', train=False, download=True, transform=transform)

    # 結果を図形表示するためのテストデータセット(加工しない:ToTensor()のみ)
    test0_set = datasets.EMNIST(root=data_root, split='balanced', train=False, download=True, transform=transforms.ToTensor())

    return train_set, test_set, test0_set
