# -*- coding: utf-8 -*-
"""
Flowers102.py
花, 102種類
"""

import os
import numpy as np
import torch
from scipy.io import loadmat
from PIL import Image

# dataset
def dataset(in_size, ud, rl, nd):
    # ラベル
    label_path = os.path.join(os.getcwd(), 'flowers-102', 'imagelabels.mat')
    imagelabels = loadmat(label_path)
    labels0 = imagelabels['labels'][0]
    labels = labels0.astype('i8').copy()
    #print(len(labels), np.min(labels), np.max(labels))  # 8189 1 102
    
    # 画像
    image_dir = os.path.join(os.getcwd(), 'flowers-102', 'jpg')
    rand = torch.rand(len(labels))
    train_ratio = 0.8  # 訓練/テスト分配比
    train_set, test_set = [], []
    i = 0
    for filename in os.listdir(image_dir):
        filepath = os.path.join(image_dir, filename)
        if os.path.isfile(filepath):
            # 画像開く
            pil = Image.open(filepath)
            # リサイズ
            pil = pil.resize((in_size[1], in_size[2]))
            # Numpy配列: 0-255 -> 0-1
            image = np.array(pil, dtype='f4') / 255
            # 転置
            image = np.transpose(image, (2, 0, 1))
            image00 = image.copy()
            image01 = np.flip(image00, axis=1).copy()  # 上下反転画像
            image10 = np.flip(image00, axis=2).copy()  # 左右反転画像
            image11 = np.flip(image01, axis=2).copy()  # 上下左右反転画像
            # tensor
            image00 = torch.tensor(image00)
            image01 = torch.tensor(image01)
            image10 = torch.tensor(image10)
            image11 = torch.tensor(image11)
            # ラベル: 開始番号を1から0に変える
            label = labels[i] - 1
            # 乱数による分配
            if rand[i] < train_ratio:
                for _ in range(nd):
                    train_set.append([image00, label])
                    if ud:
                        train_set.append([image01, label])
                    if rl:
                        train_set.append([image10, label])
                    if ud and rl:
                        train_set.append([image11, label])
            else:
                test_set.append([image, label])
            i += 1

    # テスト用データセットのコピー
    test0_set = test_set.copy()

    # debug
    #_plot_dataset(train_set)

    return train_set, test_set, test0_set

"""
# debug : データセットの確認
import matplotlib.pyplot as plt
import labels
def _plot_dataset(dataset):

    w = 9
    h = 7
    nrow = 4
    ncol = 4
    fontsize = 7
    chars = 11

    strclasses = labels.get('Flowers102')

    # 図形表示開始
    plt.figure('Flowers102 dataset', figsize=(w, h))

    # フォント(日本語表示用)
    plt.rcParams['font.family'] = 'MS gothic'

    done = np.zeros(len(strclasses), 'i4')
    ipos = 1
    for (image, label) in dataset:
        #if (done[label] == 0) and (ipos <= nrow * ncol):  # 最初の1個
        if ipos <= nrow * ncol:  # 最初の指定個数
            #print(label)
            ax = plt.subplot(nrow, ncol, ipos)
            image2 = np.transpose(image, (1, 2, 0))
            plt.imshow(image2)
            ipos += 1
            done[label] = 1
            ax.set_axis_off()
            ax.set_title(strclasses[label][:chars], fontsize=fontsize, pad=0)

if __name__ == "__main__":
    dataset((3, 100, 100), True, True)
"""
