# -*- coding: utf-8 -*-
"""
K49.py
手書きひらがな(Kuzushiji-49): 49文字
1x28x28
http://codh.rois.ac.jp/kmnist/
https://github.com/rois-codh/kmnist
"""

import numpy as np
import torch

# dataset
def dataset(MODEL, in_size):
    # 232365 + 38547 = 270912
    # 28 x 28, gray scale
    # d1.shape = (232365, 28, 28) (uint8) 0-255
    # d2.shape = (232365,)        (uint8)
    # d3.shape = (38547, 28, 28)  (uint8) 0-255
    # d4.shape = (38547,)         (uint8)
    d1 = np.load('./K49/k49-train-imgs.npz')['arr_0']
    d2 = np.load('./K49/k49-train-labels.npz')['arr_0']
    d3 = np.load('./K49/k49-test-imgs.npz')['arr_0']
    d4 = np.load('./K49/k49-test-labels.npz')['arr_0']

    assert(len(d1) == len(d2))
    assert(len(d3) == len(d4))

    # 0-255 -> 0-1
    d1 = d1.astype(np.float32) / 255.0
    d3 = d3.astype(np.float32) / 255.0

    # 白黒反転, 白地に黒字
    d1 = 1 - d1
    d3 = 1 - d3

    # 正規化(mojilaには不要)
    """
    mean1 = np.mean(d1)
    mean3 = np.mean(d3)
    std1 = np.std(d1)
    std3 = np.std(d3)
    #print(mean1, mean3, std1, std3) # 0.1800 0.1742 0.3421 0.3364
    d1 = (d1 - mean1) / std1
    d3 = (d3 - mean3) / std3
    """

    # 配列サイズ変換
    if (MODEL == 0) or (MODEL == 1):
        # [28, 28] -> [1, 28, 28]
        d1 = d1.reshape(-1, 1, in_size[1], in_size[2])
        d3 = d3.reshape(-1, 1, in_size[1], in_size[2])
    elif (MODEL == 2):
        # 公開ResNet: モノクロをカラー(R=G=B)に変換する
        d1a = np.zeros((d1.shape[0], 3, d1.shape[1], d1.shape[2]), dtype='f4')
        d3a = np.zeros((d3.shape[0], 3, d3.shape[1], d3.shape[2]), dtype='f4')
        d1a[:, 0, :, :] = d1a[:, 1, :, :] = d1a[:, 2, :, :] = d1[:, :, :]
        d3a[:, 0, :, :] = d3a[:, 1, :, :] = d3a[:, 2, :, :] = d3[:, :, :]
        d1 = d1a.copy()
        d3 = d3a.copy()

    # tensor
    d1 = torch.tensor(d1)
    d3 = torch.tensor(d3)

    #d2 = torch.tensor(d2) # 不要
    #d4 = torch.tensor(d4) # 不要

    train_set = []
    for i in range(len(d1)):
        train_set.append([d1[i], d2[i]])

    test_set = []
    for i in range(len(d3)):
        test_set.append([d3[i], d4[i]])
    
    test0_set = test_set.copy()

    return train_set, test_set, test0_set
