# -*- coding: utf-8 -*-
"""
MNIST.py
1x28x28

(1) MNIST : 手書き数字: 0-9 (60000, 10000)
https://yann.lecun.com/exdb/mnist/

(2) QMNIST :手書き数字: 0-9 (60000, 60000)
https://github.com/facebookresearch/qmnist

(3) FashionMNIST : ファッション商品: 10種類 (60000, 10000)
https://github.com/zalandoresearch/fashion-mnist

(4) KMNIST : 手書きひらがな(Kuzushiji-MNIST): 10文字 (60000, 10000)
http://codh.rois.ac.jp/kmnist/
https://github.com/rois-codh/kmnist
"""

import os
#import numpy as np
import torch
from torchvision import datasets, transforms
#import utils

# dataset
# MODEL = 0/1/2/3 : 自作CNN, 自作ResNet, 公開ResNet, VGG
def dataset(DATA, MODEL, in_size):
    # dataset
    if   DATA == 'MNIST':
        datasets_fn = datasets.MNIST
    elif DATA == 'QMNIST':
        datasets_fn = datasets.QMNIST
    elif DATA == 'FashionMNIST':
        datasets_fn = datasets.FashionMNIST
    elif DATA == 'KMNIST':
        datasets_fn = datasets.KMNIST
    else:
        print('invalid DATA : %s' % DATA)
        return

    # データファイルを保存する場所
    data_root = os.getcwd()

    # 結果を図形表示するためのテストデータセット(加工しない:ToTensor()のみ)
    test0_set = datasets_fn(root=data_root, train=False, download=True, transform=transforms.ToTensor())

    # 平均と分散
    #mean, std = utils.mean_std(test0_set, in_size[0])
    #print(mean, std)
    # MNIST : 0.1325, 0.3105
    # QMNIST : 0.1318, 0.3099
    # FashionMNIST : 0.2868, 0.3524
    # KMNIST : 0.1846, 0.3416

    # 前処理
    if (MODEL == 0) or (MODEL == 1):
        transform = transforms.Compose([
            transforms.ToTensor(),  # 必須
            # 以下必要なら追加する(オプション)
            #transforms.Normalize(mean, std),  # 正規化(MNISTでは不要)
        ])
    else:
        transform = transforms.Compose([
            transforms.ToTensor(),  # 必須
            transforms.Lambda(lambda x: torch.cat((x, x, x), 0).view(-1, in_size[1], in_size[2])), # gray->RGB
            # 以下必要なら追加する(オプション)
            #transforms.Resize(112),  # サイズ拡大(計算時間が増える,MNISTでは不要)
            #transforms.Normalize(mean, std),  # 正規化(MNISTでは不要)
        ])

    # 訓練、テスト用データセット
    train_set = datasets_fn(root=data_root, train=True,  download=True, transform=transform)
    test_set  = datasets_fn(root=data_root, train=False, download=True, transform=transform)
    #print(train_set)

    """
    # 訓練データ数削減テスト
    ndata = 10000
    rnd = np.random.rand(ndata)
    irnd = np.argsort(rnd)
    train_set_sub = []
    for i in range(ndata):
        train_set_sub.append(train_set[irnd[i]])
    return train_set_sub, test_set, test0_set
    """

    return train_set, test_set, test0_set
